#include "common/uniforms/environment.glsl"

const float etaR = 0.44;
const float etaG = 0.42;
const float etaB = 0.40;
const float fresnelPower = 2.0;
const float F = ((1.0 - etaG) * (1.0 - etaG)) / ((1.0 + etaG) * (1.0 + etaG));

vec4 calcReflection(vec3 I, vec3 normal, Material material) {
    float refIntensity = (material.fShininess / 100f);
    vec3 R = reflect(I, normalize(normal));
    vec4 env = texture(environment.envCubeMap, R);
    vec4 sky = texture(environment.skyboxCubeMap, R);
    return mix(sky, env, step(COLOR_BLACK, env)) * refIntensity;
}

vec4 calcRefraction(vec3 I, vec3 normal, Material material, vec4 reflection) {
    vec3 rfR = refract(I, normal, etaR);
    vec3 rfG = refract(I, normal, etaG);
    vec3 rfB = refract(I, normal, etaB);
    vec4 rfEnv;
    rfEnv.ra = texture(environment.envCubeMap, rfR).ra;
    rfEnv.g  = texture(environment.envCubeMap, rfG).g;
    rfEnv.b  = texture(environment.envCubeMap, rfB).b;
    vec4 rfSky;
    rfSky.ra = texture(environment.skyboxCubeMap, rfR).ra;
    rfSky.g  = texture(environment.skyboxCubeMap, rfG).g;
    rfSky.b  = texture(environment.skyboxCubeMap, rfB).b;
    vec4 rfColor = mix(rfSky, rfEnv, step(COLOR_BLACK, rfEnv));
    float ratio = F + (1.0 - F) * pow(1.0 - dot(-I, normal), fresnelPower);
    return mix(rfColor, reflection, ratio);
}
