#pragma once
#include "common/defines.glsl"
#include "common/uniforms/textures.glsl"

vec2 parallaxMapping(vec2 texCoords, vec3 viewDir)
{
    #ifdef PARALAX_MAPPNING_HIGH
    float numLayers = mix(PARALAX_MIN_LAYERS, PARALAX_MAX_LAYERS, abs(dot(DIR_Z, viewDir)));

    // calculate the size of each layer
    float layerDepth = 1.0 / numLayers;
    // depth of current layer
    float currentLayerDepth = 0.0;
    // the amount to shift the texture coordinates per layer (from vector P)
    vec2 P = viewDir.xy * PARALAX_HEIGHT_SCALE * vec2(-1, 1);
    vec2 deltaTexCoords = P / numLayers;

    // get initial values
    vec2  currentTexCoords     = texCoords;
    float currentDepthMapValue = texture(textures.height, currentTexCoords).r;

    while(currentLayerDepth < currentDepthMapValue)
    {
        // shift texture coordinates along direction of P
        currentTexCoords += deltaTexCoords;
        // get depthmap value at current texture coordinates
        currentDepthMapValue = texture(textures.height, currentTexCoords).r;
        // get depth of next layer
        currentLayerDepth += layerDepth;
    }

    // get texture coordinates before collision (reverse operations)
    vec2 prevTexCoords = currentTexCoords - deltaTexCoords;

    // get depth after and before collision for linear interpolation
    float afterDepth  = currentDepthMapValue - currentLayerDepth;
    float beforeDepth = texture(textures.height, prevTexCoords).r - currentLayerDepth + layerDepth;

    // interpolation of texture coordinates
    float weight = afterDepth / (afterDepth - beforeDepth);
    vec2 finalTexCoords = prevTexCoords * weight + currentTexCoords * (1.0 - weight);

    return finalTexCoords;
    #else
    float height =  texture(textures.height, texCoords).r;
    return texCoords - viewDir.xy * (height * PARALAX_HEIGHT_SCALE) * vec2(1,-1);
    #endif
}