#pragma once
#include "common/uniforms/shadow_buffer.glsl"

float calcShadowFactorFromDirectionalLight(vec3 eyeDirection, vec4 vPositionInLightSpace, int lightType)
{
    // perform perspective divide
    vec3 projCoords = vPositionInLightSpace.xyz / vPositionInLightSpace.w;
    // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    if(projCoords.z > 1.0) return 1.0;

    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // Check whether current frag pos is in shadow
    float bias = lightType == LIGHT_TYPE_SUN ? SHADOW_BIAS_SUN: SHADOW_BIAS_POINT;
//    float bias = max(0.05 * (1.0 - dot(vNormal, lightDir)), 0.005);
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowBuffer.flatTexture, 0);

    int samples = 0;
    float step = max(length(eyeDirection) / 10f, 0.7f);
    for(float x = -1f; x <= 1f; x += step)
    {
        for(float y = -1f; y <= 1f; y += step)
        {
            samples++;
            // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
            float pcfDepth = texture(shadowBuffer.flatTexture, projCoords.xy + vec2(x, y) * texelSize).r;
            if(currentDepth - bias > pcfDepth) {
                shadow += clamp(1.0 - pow(sqrt(currentDepth), 20) + 0.4, 0.0, 1.0);
            }
        }
    }

    shadow /= samples;
    return 1.0 - shadow;
}

const int GRID_SAMPLES = 6;
// array of offset direction for sampling
vec3 gridSamplingDisk[GRID_SAMPLES] = vec3[]
(
   vec3(0, 1, 0), /*vec3(1, -1, 1), */vec3(0, 0, 1), /*vec3(-1, 1, 1),*/
   vec3(1, 0, 0), /*vec3(1, -1, -1),*/ vec3(0, 0, -1), /*vec3(-1, 1, -1),*/
   vec3(-1, 0, 0), /*vec3(1, -1, 0), vec3(0, 0, 0), vec3(-1, 1, 0),*/
   vec3(0, -1, 0)/*, vec3(-1, 0, 1), vec3(0, 0, 0), vec3(-1, 0, -1),*/
   /*vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)*/
);

float calcShadowFactorFromPointLight(vec3 eyeDirection, vec3 lightDirection, float shadowDistance)
{
   float shadow = 0.0;
   float bias = SHADOW_BIAS_POINT;
   int samples = GRID_SAMPLES;
   float viewDistance = length(eyeDirection) / shadowDistance; // mapping [0;1]
   float currentDepth = length(lightDirection) / shadowDistance; // mapping [0;1]
   float diskRadius = (1.0 + viewDistance) / 25.0;

   for(int i = 0; i < samples; ++i) {
       float closestDepth = texture(shadowBuffer.cubeMapTexture, lightDirection + gridSamplingDisk[i] * diskRadius).r;
       if(currentDepth - bias > closestDepth) {
           shadow += clamp(1.0 - pow(sqrt(currentDepth), 20), 0.0, 1.0);
       }
   }

   shadow /= float(samples);
   return 1.0 - shadow;
}
